#include <windows.h>
#include <scrnsave.h>
#include <time.h>
#include <math.h>
#include "mathlocal.h"
#include "resource.h"

#define REGENTRY "Software\\matobaa\\SkyRootScreenSaver"

int		multi_color = TRUE;
int		cloudy = FALSE;
double		view_angle = 54, view_center = 270;
double		curr_lon=139.74469, curr_lat=35.654;
int		view_horizon_dots = 0;
int		v_cell_dots = 8;
int		h_cell_dots = 400;
BOOL	demoMode = FALSE;
static HKEY hKey = NULL;
static HWND angleSlider;
static HWND directionSlider;

extern void SkyInit(RECT rect);
extern void sunpos_local(
    struct tm *t_gmt,			/* current gmt time */
    double curr_lon,
    double curr_lat,		/* longitude/latitude on earth */
    double *p_sun_hour,
    double *p_sun_lon,
    double *p_sun_lat,
    double *px, double *py, double *pz);

extern void SkyUpdate( HWND hWnd, double lon, double z );

DWORD loadValuesAsDWORD(char *valueName, DWORD defaultValue) {
    DWORD data = defaultValue;
    DWORD size = sizeof(DWORD);
    if(hKey==NULL) {
	RegCreateKeyEx(HKEY_CURRENT_USER, REGENTRY, 0, NULL,
	    REG_OPTION_NON_VOLATILE, KEY_READ, NULL,
	    &hKey, NULL);
	}
    RegQueryValueEx(hKey, valueName,0,NULL, (UCHAR*)&data, &size);
    return data;
}

DWORD saveValues(char* valueName, DWORD value) {
    if(hKey==NULL) {
	RegCreateKeyEx(HKEY_CURRENT_USER, REGENTRY, 0, NULL,
	    REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL,
	    &hKey, NULL);
	}
    RegSetValueEx(hKey, valueName,0,REG_DWORD, (BYTE*)&value, sizeof(DWORD));
    return value;
}

static SCROLLINFO sinfo;
const int step=30;

BOOL setAngle(HWND hWnd, double angle) {
	sinfo.nPos=(int)angle; sinfo.fMask=SIF_POS;
	SetScrollInfo(angleSlider,SB_CTL,&sinfo,TRUE);
	SetDlgItemInt(hWnd,IDC_ANGLE,(int)angle,FALSE);
	return TRUE;
}
	
BOOL setDirection(HWND hWnd, double direction) {
	sinfo.nPos=(int)direction; sinfo.fMask=SIF_POS;
	SetScrollInfo(directionSlider,SB_CTL,&sinfo,TRUE);
	SetDlgItemInt(hWnd,IDC_DIRECTION,(int)direction,FALSE);
	return TRUE;
}

double validate(double src, int min, int max) {
	return src < min ? min : src > max ? max : src;
}

void loadValues() {
    view_angle = loadValuesAsDWORD("angle",36);
    view_center = loadValuesAsDWORD("direction",270);
	view_horizon_dots = loadValuesAsDWORD("horizon",0);
	multi_color = !loadValuesAsDWORD("solid",FALSE);
	cloudy = loadValuesAsDWORD("cloudy",FALSE);
}

void reflectValues(HWND hWnd) {
	setAngle(hWnd,view_angle);
	setDirection(hWnd,view_center);
	SetDlgItemInt(hWnd,IDC_HORIZON,view_horizon_dots,TRUE);
	CheckDlgButton(hWnd,IDC_SOLID,!multi_color?BST_CHECKED:BST_UNCHECKED);
	CheckDlgButton(hWnd,IDC_CLOUD,cloudy?BST_CHECKED:BST_UNCHECKED);
}

BOOL WINAPI ScreenSaverConfigureDialog(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam) {
    BOOL result;
    result = FALSE;
    switch(uMsg) {
    case WM_INITDIALOG:
	loadValues();
	SetDlgItemText(hWnd,IDC_CAPTION,"http://hp.vector.co.jp/authors/VA025338/");
	SetDlgItemText(hWnd,IDC_LOCATION,"|̐ݒʂ͂܂g܂|");
	angleSlider = GetDlgItem(hWnd,IDC_AngleSlider);
	directionSlider = GetDlgItem(hWnd,IDC_DirectionSlider);
	sinfo.cbSize = sizeof(SCROLLINFO);
	sinfo.nMin=1; sinfo.nMax=360;sinfo.fMask=SIF_RANGE;
	SetScrollInfo(angleSlider,SB_CTL,&sinfo,FALSE);
	sinfo.nMin=0; sinfo.nMax=360;sinfo.fMask=SIF_RANGE;
	SetScrollInfo(directionSlider,SB_CTL,&sinfo,FALSE);
	reflectValues(hWnd);
	RegCloseKey(hKey);hKey=NULL;
	result=TRUE;
	break;
    case WM_HSCROLL:
	if((HWND)lParam==angleSlider) {
		int angle = LOWORD(wParam)==SB_THUMBTRACK ? HIWORD(wParam) :
				wParam==SB_PAGEDOWN ? (int)view_angle+step :
				wParam==SB_PAGEUP ? (int)view_angle-step :
				wParam==SB_LINEDOWN ? (int)view_angle + 1 :
				wParam==SB_LINEUP ? (int)view_angle - 1 :
				wParam==SB_TOP ? 1 :
				wParam==SB_BOTTOM ? 360 : (int)view_angle ;
		view_angle = validate(angle,1,360);
		result = setAngle(hWnd, view_angle);
	} else if((HWND)lParam==directionSlider) {
		int direction = LOWORD(wParam)==SB_THUMBTRACK ? HIWORD(wParam) :
				wParam==SB_PAGEDOWN ? (int)view_center+step :
				wParam==SB_PAGEUP ? (int)view_center-step :
				wParam==SB_LINEDOWN ? (int)view_center + 1 :
				wParam==SB_LINEUP ? (int)view_center - 1 :
				wParam==SB_TOP ? 0 :
				wParam==SB_BOTTOM ? 360 : (int)view_center ;
		view_center = validate(direction, 0, 360);
	    result = setDirection(hWnd, view_center);
	}
	break;
    case WM_COMMAND:
	switch(LOWORD(wParam)){
	case IDOK:
	    saveValues("angle",(int)view_angle);
	    saveValues("direction",(int)view_center);
		saveValues("horizon",GetDlgItemInt(hWnd,IDC_HORIZON,NULL,FALSE));
		saveValues("solid",IsDlgButtonChecked(hWnd,IDC_SOLID) == BST_CHECKED);
		saveValues("cloudy",IsDlgButtonChecked(hWnd,IDC_CLOUD) == BST_CHECKED);
		RegCloseKey(hKey);hKey=NULL;
	    /* fall through... */
	case IDCANCEL:
	    EndDialog(hWnd,IDCANCEL);
	    result=TRUE;
	    break;
	case IDDefault:
		// TODO: default setting
		break;
	}

    default:
	break;
    }
    return result;
}

LONG WINAPI ScreenSaverProc(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam) {

    static RECT rectClient;
    double hdash,lon,lat, x, y, z;
    struct tm *t_gmt;
    long tval;
	switch(uMsg){
    case WM_DESTROY:
	KillTimer(hWnd,1);
	break;
    case WM_CREATE:
	GetClientRect(hWnd,&rectClient);
	loadValues();
	demoMode=(rectClient.bottom<400);
    SkyInit( rectClient );
	if(demoMode) curr_lon=0;
	SetTimer(hWnd,1,demoMode?100:1000,NULL);
	/* fall through... */
    case WM_TIMER:
	time(&tval);
	t_gmt = gmtime(&tval);
	if(demoMode) curr_lon=((int)curr_lon+180)%360-177;
	sunpos_local
	    ( t_gmt, curr_lon, curr_lat, &hdash,&lon,&lat, &x,&y,&z );
	SkyUpdate( hWnd, fixangle(lon), z);
	break;
    default:
	return DefScreenSaverProc(hWnd,uMsg,wParam,lParam);
    }
    return 0;
}

BOOL WINAPI RegisterDialogClasses(HANDLE hInst){ return TRUE; }

/*
parse_locate(s,lon,lat)
    char *s;
    double *lon, *lat;
{
    double k, *x;

    // locate format: Ann/Bnnn

    while( *s ) {
	while( *s=='/' )
	    s++;
	switch( *s++ ) {
	case 'N':
	case 'n':
	    x=lat; k = 1; break;
	case 'S':
	case 's':
	    x=lat; k = -1; break;
	case 'E':
	case 'e':
	    x=lon; k = 1; break;
	case 'W':
	case 'w':
	    x=lon; k = -1; break;
	default:
	    ;
	}
	if(x)
	    *x = k * atof(s);
	while( *s && *s != '/' )
	    s++;
    }
}

*/
